package net.sourceforge.jpowergraph.painters;

import net.sourceforge.jpowergraph.Node;
import net.sourceforge.jpowergraph.manipulator.dragging.DraggingManipulator;
import net.sourceforge.jpowergraph.manipulator.selection.HighlightingManipulator;
import net.sourceforge.jpowergraph.manipulator.selection.SelectionManipulator;
import net.sourceforge.jpowergraph.pane.JGraphPane;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;


/**
 * The painter drawing the node as a rectangle.
 */
public class ShapeNodePainter implements NodePainter {

    public static final int RECTANGLE = 0;
    public static final int ELLIPSE = 1;
    public static final int TRIANGLE = 2;
    
    private Color backgroundColor;
    private Color borderColor;
    private Color textColor;
    private FontMetrics fontMetrics;
    private int shape;
    private Composite parent;
    private GC g;
    
    private boolean disposed = false;
    
    public ShapeNodePainter (Composite theParent, int theShape){
        this(theParent, theShape, theParent.getDisplay().getSystemColor(SWT.COLOR_GRAY), theParent.getDisplay().getSystemColor(SWT.COLOR_DARK_GRAY), theParent.getDisplay().getSystemColor(SWT.COLOR_BLACK));
    }
    
    public ShapeNodePainter (Composite theParent, int theShape, Color theBackgroundColor, Color theBorderColor, Color theTextColor){
        this.parent = theParent;
        this.shape = theShape;
        this.backgroundColor = theBackgroundColor;
        this.borderColor = theBorderColor;
        this.textColor = theTextColor;
    }

    /**
     * Paints the supplied node.
     *
     * @param graphPane             the graph pane
     * @param g                     the graphics
     * @param node                  the node to paint
     */
    public void paintNode(JGraphPane graphPane,GC g,Node node, int size) {
        HighlightingManipulator highlightingManipulator=(HighlightingManipulator)graphPane.getManipulator(HighlightingManipulator.NAME);
        boolean isHighlighted=highlightingManipulator!=null && highlightingManipulator.getHighlightedNode()==node;
        SelectionManipulator selectionManipulator=(SelectionManipulator)graphPane.getManipulator(SelectionManipulator.NAME);
        boolean isSelected=selectionManipulator!=null && selectionManipulator.getNodeSelectionModel().isNodeSelected(node);
        DraggingManipulator draggingManipulator=(DraggingManipulator)graphPane.getManipulator(DraggingManipulator.NAME);
        boolean isDragging=draggingManipulator!=null && draggingManipulator.getDraggedNode()==node;
        
        if (fontMetrics == null){
            fontMetrics = g.getFontMetrics();
        }
        
        Point nodePoint=graphPane.getScreenPointForNode(node);
        if (size == NodePainter.LARGE){ 
            int width=20;
            int height=5;
            int textX=0;
            int textY=0;
            String label=node.getLabel();
            if (label!=null) {
                int stringWidth = stringWidth(g, label);
                width+=stringWidth+10;
                height+=fontMetrics.getAscent()+fontMetrics.getDescent()+8;
                textX=nodePoint.x-stringWidth/2;
                textY=nodePoint.y - 7;
                if (shape == TRIANGLE){
                    textY += 7;
                }
            }
            else {
                width+=40;
                height+=20;
            }
            Color oldBGColor=g.getBackground();
            Color oldFGColor=g.getForeground();
            g.setBackground(getBackgroundColor(isHighlighted,isSelected,isDragging));
            if (shape == RECTANGLE){
            	g.fillRectangle(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == ELLIPSE){
            	g.fillOval(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == TRIANGLE){
                int x1 = nodePoint.x - width/2;
                int y1 = nodePoint.y + height/2;
                int x2 = x1 + width/2;
                int y2 = nodePoint.y - height/2;
                int x3 = x1 + width;
                int y3 = y1;
                
                g.fillPolygon(new int[]{x1, y1, x2, y2, x3, y3});
            }
            if (label!=null) {
                Font oldFont=g.getFont();
                g.setFont(graphPane.getFont());
                g.setForeground(getTextColor(isHighlighted,isSelected,isDragging));
                g.drawString(label,textX,textY);
                g.setFont(oldFont);
            }
            g.setForeground(getBorderColor(isHighlighted,isSelected,isDragging));
            if (shape == RECTANGLE){
            	g.drawRectangle(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == ELLIPSE){
            	g.drawOval(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == TRIANGLE){
                int x1 = nodePoint.x - width/2;
                int y1 = nodePoint.y + height/2;
                int x2 = x1 + width/2;
                int y2 = nodePoint.y - height/2;
                int x3 = x1 + width;
                int y3 = y1;
                
                g.drawPolygon(new int[]{x1, y1, x2, y2, x3, y3});
            }
            g.setBackground(oldBGColor);
            g.setForeground(oldFGColor);
        }
        else if (size == NodePainter.SMALL){
            int width=18;
            int height=8;
            int textX = nodePoint.x + width;
            int textY = nodePoint.y - 7;
            String label=node.getLabel();
            Color oldBGColor=g.getBackground();
            Color oldFGColor=g.getForeground();
            g.setBackground(getBackgroundColor(isHighlighted,isSelected,isDragging));
            if (shape == RECTANGLE){
                g.fillRectangle(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == ELLIPSE){
                g.fillOval(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == TRIANGLE){
                int x1 = nodePoint.x - width/2;
                int y1 = nodePoint.y + height/2;
                int x2 = x1 + width/2;
                int y2 = nodePoint.y - height/2;
                int x3 = x1 + width;
                int y3 = y1;
                
                g.fillPolygon(new int[]{x1, y1, x2, y2, x3, y3});
            }
            g.setBackground(oldBGColor);
            if (label!=null) {
                Font oldFont=g.getFont();
                g.setFont(graphPane.getFont());
                g.setForeground(getTextColor(isHighlighted,isSelected,isDragging));
                g.drawString(label,textX,textY);
                g.setFont(oldFont);
            }
            g.setForeground(getBorderColor(isHighlighted,isSelected,isDragging));
            if (shape == RECTANGLE){
                g.drawRectangle(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == ELLIPSE){
                g.drawOval(nodePoint.x-width/2,nodePoint.y-height/2,width,height);
            }
            else if (shape == TRIANGLE){
                int x1 = nodePoint.x - width/2;
                int y1 = nodePoint.y + height/2;
                int x2 = x1 + width/2;
                int y2 = nodePoint.y - height/2;
                int x3 = x1 + width;
                int y3 = y1;
                
                g.drawPolygon(new int[]{x1, y1, x2, y2, x3, y3});
            }
            g.setBackground(oldBGColor);
            g.setForeground(oldFGColor);
        }
    }
    /**
     * Returns the border color of the node.
     *
     * @param isHighlighted         <code>true</code> if the node is highlighted
     * @param isSelected            <code>true</code> if the node is selected
     * @param isDragging            <code>true</code> if the node is being dragged
     * @return                      the border color
     */
    public Color getBorderColor(boolean isHighlighted,boolean isSelected,boolean isDragging) {
        if (isHighlighted || isDragging || isSelected){
            return backgroundColor;
        }
        return borderColor;
    }
    /**
     * Returns the background color of the node.
     *
     * @param isHighlighted         <code>true</code> if the node is highlighted
     * @param isSelected            <code>true</code> if the node is selected
     * @param isDragging            <code>true</code> if the node is being dragged
     * @return                      the background color
     */
    public Color getBackgroundColor(boolean isHighlighted,boolean isSelected,boolean isDragging) {
        if (isHighlighted || isDragging || isSelected){
            return borderColor;
        }
        return backgroundColor;
    }
    /**
     * Returns the text color of the node
     *
     * @param isHighlighted         <code>true</code> if the node is highlighted
     * @param isSelected            <code>true</code> if the node is selected
     * @param isDragging            <code>true</code> if the node is being dragged
     * @return                      the background color
     */
    public Color getTextColor(boolean isHighlighted,boolean isSelected,boolean isDragging) {
        if (isHighlighted || isDragging || isSelected){
            return textColor;
        }
        return textColor;
    }
    /**
     * Checks whether given point is inside the node.
     *
     * @param graphPane             the graph pane
     * @param node                  the node
     * @param point                 the point
     * @return                      <code>true</code> if the point is in the node
     */
    public boolean isInNode(JGraphPane graphPane,Node node,Point point, int size) {
        Rectangle nodeScreenRectangle=new Rectangle(0, 0, 0, 0);
        getNodeScreenBounds(graphPane,node, size, nodeScreenRectangle);
        return nodeScreenRectangle.contains(point);
    }
    
    /**
     * Returns the outer rectangle of the node on screen.
     *
     * @param graphPane             the graph pane
     * @param node                  the node
     * @param nodeScreenRectangle   the rectangle receiving the node's coordinates
     */
    public void getNodeScreenBounds(JGraphPane graphPane,Node node, int size, Rectangle nodeScreenRectangle) {
        Point nodePoint=graphPane.getScreenPointForNode(node);
        String label=node.getLabel();
        
        if (g == null){
            Image im = new Image(parent.getDisplay(), 100, 100); 
            g = new GC(im);
        }
        
        if (fontMetrics == null){
            fontMetrics = g.getFontMetrics();
        }
        
        if (size == NodePainter.LARGE){ 
            int width=1;
            int height=1;
            if (label!=null) {
                width+=stringWidth(g, label)+6;
                height+=fontMetrics.getAscent() + fontMetrics.getDescent()+4;
            }
            else {
                width+=40;
                height+=20;
            }
            
            nodeScreenRectangle.x = nodePoint.x-width/2;
            nodeScreenRectangle.y = nodePoint.y-height/2;
            nodeScreenRectangle.width = width;
            nodeScreenRectangle.height = height;
        }
        else if (size == NodePainter.SMALL){
            int width = 20 + stringWidth(g, label);
            int height = 8;
            nodeScreenRectangle.x = nodePoint.x-width/2;
            nodeScreenRectangle.y = nodePoint.y-height/2;
            nodeScreenRectangle.width = width;
            nodeScreenRectangle.height = height;
        }
    }
    
    private int stringWidth(GC g, String s){
        return g.stringExtent(s).x;
    }
    
    /**
     * Retruns the tool-tip for given point.
     *
     * @param graphPane             the graph pane
     * @param node                  the node
     * @param point                 the point
     * @return                      the tool-tip at given point (or <code>null</code>)
     */
    public String getToolTipText(JGraphPane graphPane,Node node,Point point) {
        return null;
    }
    
    public void paintLegendItem(JGraphPane graphPane, GC g, Point thePoint, String legendText){
        if (fontMetrics == null){
            fontMetrics = g.getFontMetrics();
        }
        
        int padding = 2;
        int imageWidth = 18;
        int imageHeight = 8;
        int imageX = thePoint.x;
        int imageY = thePoint.y; 
        int textX = imageX + imageWidth + (padding * 3);
        int textY = imageY - 3;
        
        Color bgColor = getBackgroundColor(false, false, false);
        Color boColor = getBorderColor(false, false, false);
        Color teColor = getTextColor(false, false, false);
        
        Color oldFGColor = g.getForeground();
        Color oldBGColor = g.getBackground();
        if (shape == RECTANGLE){
            g.setBackground(bgColor);
            g.fillRectangle(imageX, imageY, imageWidth, imageHeight);
            g.setForeground(boColor);
            g.drawRectangle(imageX, imageY, imageWidth, imageHeight);
        }
        else if (shape == ELLIPSE){
            g.setBackground(bgColor);
            g.fillOval(imageX, imageY, imageWidth, imageHeight);
            g.setForeground(boColor);
            g.drawOval(imageX, imageY, imageWidth, imageHeight);
        }
        else if (shape == TRIANGLE){
            int x1 = imageX;
            int y1 = imageY + imageHeight;
            int x2 = x1 + imageWidth/2;
            int y2 = imageY - imageHeight/2;
            int x3 = x1 + imageWidth;
            int y3 = y1;
            
            g.setBackground(bgColor);
            g.fillPolygon(new int[]{x1, y1, x2, y2, x3, y3});
            g.setForeground(boColor);
            g.drawPolygon(new int[]{x1, y1, x2, y2, x3, y3});
        }
        g.setBackground(oldBGColor);
        g.setForeground(teColor);
        g.drawString(legendText, textX, textY);
        g.setForeground(oldFGColor);
        g.setBackground(oldBGColor);
    }
    
    public Point getLegendItemSize(JGraphPane graphPane, String legendText){
        if (g == null){
            Image im = new Image(parent.getDisplay(), 100, 100); 
            g = new GC(im);
        }
        
        if (fontMetrics == null){
            fontMetrics = g.getFontMetrics();
        }
        
        int padding = 2;
        int imageWidth = 18;
        int imageHeight = 8;
        int stringWidth = stringWidth(g, legendText);
        int width = imageWidth + stringWidth + (padding*3);
        int height= Math.max(imageHeight, fontMetrics.getAscent() + fontMetrics.getDescent() + 4);
        return new Point(width, height);
    }
    
    public Color getBorderColor() {
        return borderColor;
    }
    
    public Color getBackgroundColor() {
        return backgroundColor;
    }
    
    public void dispose(){
        if (!backgroundColor.isDisposed()){
            backgroundColor.dispose();
        }
        if (!borderColor.isDisposed()){
            borderColor.dispose();
        }
        if (!textColor.isDisposed()){
            textColor.dispose();
        }
        disposed = true;
    }
    
    public boolean isDisposed(){
        return disposed;
    }
}
